"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.QueueFlushingPlugin = void 0;

var _sovranReactNative = require("@segment/sovran-react-native");

var _constants = require("../constants");

var _plugin = require("../plugin");

var _types = require("../types");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * This plugin manages a queue where all events get added to after timeline processing.
 * It takes a onFlush callback to trigger any action particular to your destination sending events.
 * It can autotrigger a flush of the queue when it reaches the config flushAt limit.
 */
class QueueFlushingPlugin extends _plugin.UtilityPlugin {
  // Gets executed last to keep the queue after all timeline processing is done

  /**
   * @param onFlush callback to execute when the queue is flushed (either by reaching the limit or manually) e.g. code to upload events to your destination
   */
  constructor(onFlush) {
    let storeKey = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'events';
    super();

    _defineProperty(this, "type", _types.PluginType.after);

    _defineProperty(this, "storeKey", void 0);

    _defineProperty(this, "isPendingUpload", false);

    _defineProperty(this, "queueStore", void 0);

    _defineProperty(this, "onFlush", void 0);

    this.onFlush = onFlush;
    this.storeKey = storeKey;
  }

  configure(analytics) {
    super.configure(analytics);
    const config = (analytics === null || analytics === void 0 ? void 0 : analytics.getConfig()) ?? _constants.defaultConfig; // Create its own storage per SegmentDestination instance to support multiple instances

    this.queueStore = (0, _sovranReactNative.createStore)({
      events: []
    }, {
      persist: {
        storeId: `${config.writeKey}-${this.storeKey}`,
        persistor: config.storePersistor,
        saveDelay: config.storePersistorSaveDelay ?? 0
      }
    });
  }

  async execute(event) {
    var _this$queueStore;

    await ((_this$queueStore = this.queueStore) === null || _this$queueStore === void 0 ? void 0 : _this$queueStore.dispatch(state => {
      const events = [...state.events, event];
      return {
        events
      };
    }));
    return event;
  }
  /**
   * Calls the onFlush callback with the events in the queue
   */


  async flush() {
    var _await$this$queueStor, _this$queueStore2;

    const events = ((_await$this$queueStor = await ((_this$queueStore2 = this.queueStore) === null || _this$queueStore2 === void 0 ? void 0 : _this$queueStore2.getState(true))) === null || _await$this$queueStor === void 0 ? void 0 : _await$this$queueStor.events) ?? [];

    if (!this.isPendingUpload) {
      try {
        this.isPendingUpload = true;
        await this.onFlush(events);
      } finally {
        this.isPendingUpload = false;
      }
    }
  }
  /**
   * Removes one or multiple events from the queue
   * @param events events to remove
   */


  async dequeue(events) {
    var _this$queueStore3;

    await ((_this$queueStore3 = this.queueStore) === null || _this$queueStore3 === void 0 ? void 0 : _this$queueStore3.dispatch(state => {
      const eventsToRemove = Array.isArray(events) ? events : [events];

      if (eventsToRemove.length === 0 || state.events.length === 0) {
        return state;
      }

      const setToRemove = new Set(eventsToRemove);
      const filteredEvents = state.events.filter(e => !setToRemove.has(e));
      return {
        events: filteredEvents
      };
    }));
  }

}

exports.QueueFlushingPlugin = QueueFlushingPlugin;
//# sourceMappingURL=QueueFlushingPlugin.js.map